import { type APIRoute } from "astro";
import validateAccessToken from "../../../lib/utils/validateAccessToken";
import { getUserResources, updateUserResources } from "../../../lib/utils/resourceManager";
import research from '../../../lib/data/research.json';
import { createOrUpgradeResearch, getUserByAccessToken, getUserResearch } from "../../../lib/db/users";
import type DBResource from "../../../types/DBResource";

export const POST: APIRoute = async({ request }) => {
    // validate access token
    const response = await validateAccessToken(request);
    if(response instanceof Response) return response;

    const user = await getUserByAccessToken(response);
    if(user === null) {
        return new Response(
            JSON.stringify({
                code: 401,
                message: "Unauthorized"
            }), { status: 401 }
        )
    }

    // check if research id is valid
    const researchId = (await request.json()).research;
    const researchData = research.filter((element: any) => element.id === researchId)[0];
    if(!researchId || !researchData) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid research id"
            }), { status: 400 }
        )
    }
    
    // check if user has enough resources
    const resources = await getUserResources(user._id);
    const playerResearch = await getUserResearch(user);
    const playerCurrentResearch = playerResearch.filter((element: any) => element.id === researchId)[0];
    const level = playerCurrentResearch ? playerCurrentResearch.level : 0;
    const newResources = structuredClone(resources);
    let hasEnoughResources = true;
    Object.entries(researchData.cost).forEach(([key, value]) => {
        const res = resources.filter((element: DBResource) => element.name === key)[0];
        const cost = playerCurrentResearch ? value * Math.pow(researchData.multiplier, level) : value;
        
        if(res.amount < cost) hasEnoughResources = false;
        else newResources.filter((element: DBResource) => element.name === key)[0].amount -= cost;
    });

    if(!hasEnoughResources) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Not enough resources"
            }), { status: 400 }
        )
    }

    await updateUserResources(user._id, newResources);
    await createOrUpgradeResearch(user.username, { id: researchId, level: level + 1 });

    return new Response(
        JSON.stringify({
            code: 200,
            message: "OK"
        }), { status: 200 }
    )
}